import time
import subprocess
import os
import torch
from transformers import Trainer, TrainingArguments, AutoTokenizer, AutoModelForCausalLM, DataCollatorForLanguageModeling, TextDataset

torch.cuda.empty_cache()

class TrainingMonitor(Trainer):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.start_time = time.time()
        self.last_logged_time = time.time()
        
        # 创建日志文件夹
        os.makedirs("./logs", exist_ok=True)
        self.log_file = "./logs/training_log.txt"
        
        # 写入表头
        with open(self.log_file, "w") as f:
            f.write("Step\tLoss\tLR\tThroughput\tGPU Memory Used\tGPU Memory Free\tPower Draw\tTotal Time\tCost\n")

    def log(self, logs: dict) -> None:
        try:
            current_time = time.time()
            step_time = current_time - self.last_logged_time
            self.last_logged_time = current_time

            if logs:
                # 获取基本指标
                loss = logs.get('loss', 0)
                lr = logs.get('learning_rate', 0)
                step = self.state.global_step
                epoch = logs.get('epoch', 0)
                
                # 计算额外指标
                throughput = self.args.per_device_train_batch_size / step_time if step_time > 0 else 0
                gpu_info = self.get_gpu_usage()
                total_time = current_time - self.start_time
                cost = total_time / 3600 * 0.50

                # 记录到文件
                with open(self.log_file, "a") as f:
                    f.write(f"{step}\t{loss:.4f}\t{lr:.2e}\t{throughput:.2f}\t{gpu_info['used']}\t{gpu_info['free']}\t{gpu_info['power']}\t{total_time:.2f}\t${cost:.2f}\n")

                # 打印到控制台
                print(f"Step: {step} (Epoch: {epoch:.2f})")
                print(f"Loss: {loss:.4f}, LR: {lr:.2e}, Throughput: {throughput:.2f} samples/sec")
                print(f"GPU Memory Used: {gpu_info['used']}MB, Free: {gpu_info['free']}MB, Power: {gpu_info['power']}W")
                print(f"Total Time: {total_time:.2f}s, Cost: ${cost:.2f}")
                print("-" * 80)

        except Exception as e:
            print(f"Error in logging: {str(e)}")
            
        super().log(logs)

    def get_gpu_usage(self):
        try:
            result = subprocess.run(
                ["nvidia-smi", "--query-gpu=memory.used,memory.free,power.draw", "--format=csv,noheader,nounits"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                check=True,
                timeout=5
            )
            output = result.stdout.decode('utf-8').strip()
            memory_used, memory_free, power_draw = output.split(',')
            return {
                "used": memory_used.strip(),
                "free": memory_free.strip(),
                "power": power_draw.strip()
            }
        except Exception as e:
            return {"used": 0, "free": 0, "power": 0}

# 加载模型和数据
tokenizer = AutoTokenizer.from_pretrained("anonymous-german-nlp/german-gpt2")
model = AutoModelForCausalLM.from_pretrained("anonymous-german-nlp/german-gpt2")

train_path = 'train_dataset.txt'
test_path = 'test_dataset.txt'

data_collator = DataCollatorForLanguageModeling(
    tokenizer=tokenizer,
    mlm=False
)

# 数据加载
def load_dataset(train_path, test_path, tokenizer):
    train_dataset = TextDataset(
        tokenizer=tokenizer,
        file_path=train_path,
        block_size=128
    )

    test_dataset = TextDataset(
        tokenizer=tokenizer,
        file_path=test_path,
        block_size=128
    )

    return train_dataset, test_dataset, data_collator

train_dataset, test_dataset, data_collator = load_dataset(train_path, test_path, tokenizer)

# 训练参数设置
training_args = TrainingArguments(
    output_dir="./gpt2-gerchef",
    overwrite_output_dir=True,
    num_train_epochs=1,
    per_device_train_batch_size=32,
    per_device_eval_batch_size=64,
    eval_steps=400,
    save_steps=800,
    warmup_steps=500,
    logging_steps=1,
    report_to="none"  # 禁用默认的日志记录器
)

# 训练过程监控
trainer = TrainingMonitor(
    model=model,
    args=training_args,
    data_collator=data_collator,
    train_dataset=train_dataset,
    eval_dataset=test_dataset
)

# 开始训练
print("=== Training Started ===")
trainer.train()
trainer.save_model()