# 导入所需的库
from transformers import pipeline, AutoConfig, AutoTokenizer, AutoModelForCausalLM
import torch  # 用于检查是否有可用的 GPU
# 1. 加载模型配置
# `AutoConfig.from_pretrained` 会加载指定路径下的预训练配置文件。
# 配置文件包含模型的结构、参数和其他重要信息。你可以通过修改配置来自定义模型的行为。
config = AutoConfig.from_pretrained("./gpt2-gerchef")
# 2. 加载 tokenizer 和模型
# `AutoTokenizer.from_pretrained` 会加载预训练的 tokenizer，`tokenizer` 用于将输入文本转化为模型能理解的输入格式。
# 这里加载的是一个名为 `anonymous-german-nlp/german-gpt2` 的 tokenizer，这是针对德语训练的 GPT-2 模型。
tokenizer = AutoTokenizer.from_pretrained('anonymous-german-nlp/german-gpt2')
# `AutoModelForCausalLM.from_pretrained` 用于加载预训练的 GPT-2 模型，并根据加载的配置进行初始化。
# 这里使用的是本地路径 `./gpt2-gerchef`，即加载存储在本地的 GPT-2 模型。
model = AutoModelForCausalLM.from_pretrained("./gpt2-gerchef", config=config)
# 3. 创建生成模型的 pipeline
# `pipeline` 是 Hugging Face 提供的一个简化接口，用于便捷地进行模型推理。
# 'text-generation' 表示我们希望用该 pipeline 进行文本生成任务。
# `model` 和 `tokenizer` 参数指示要使用的模型和 tokenizer。
# `device=0 if torch.cuda.is_available() else -1`：检查是否有可用的 GPU，如果有则使用 GPU（`device=0`），
# 如果没有 GPU，则使用 CPU（`device=-1`）。
# `pad_token_id=50256`：GPT-2 的 EOS（结束）标记的 token ID，告诉模型在生成时如何处理填充。
chef = pipeline('text-generation',
                model=model,                # 指定模型
                tokenizer=tokenizer,        # 指定tokenizer
                config=config,              # 传递配置
                device=0 if torch.cuda.is_available() else -1,  # 如果有GPU可用则使用GPU，否则使用CPU
                pad_token_id=50256)         # 设置填充token ID，避免生成时遇到问题
# 4. 使用生成模型进行文本生成
# 调用 pipeline，传入初始文本 `Zuerst Tomaten`，该文本作为模型的输入。模型将基于输入生成相关的文本。
# 生成的文本保存在 `result` 变量中。我们获取生成文本的第一个元素 `[0]['generated_text']`。
result = chef('Zuerst Tomaten')[0]['generated_text']
# 5. 输出生成的文本
# 打印模型生成的文本结果。
print(result)
