from transformers import AutoTokenizer, Trainer, TrainingArguments, AutoModelForCausalLM
from transformers import DataCollatorForLanguageModeling
from transformers import TextDataset

# 1. 加载预训练的Tokenizer（分词器）
tokenizer = AutoTokenizer.from_pretrained("anonymous-german-nlp/german-gpt2")

# 2. 加载预训练的GPT-2模型，模型类型为CausalLM（因其适用于语言模型训练）
model = AutoModelForCausalLM.from_pretrained("anonymous-german-nlp/german-gpt2")

# 3. 训练数据集的路径，假设 'train_dataset.txt' 是已经准备好的训练文本文件
train_path = 'train_dataset.txt'
test_path = 'test_dataset.txt'

# 4. 数据整理器，主要用于将输入文本批量处理成适合GPT模型的格式
# GPT模型是基于自回归的语言模型（Causal LM），因此我们使用 `DataCollatorForLanguageModeling` 来处理数据
data_collator = DataCollatorForLanguageModeling(
    tokenizer=tokenizer,  # 使用之前加载的分词器
    mlm=False  # GPT是自回归模型，不使用掩蔽语言建模（MLM），因此设置为 False
)

# 5. 设置训练参数
training_args = TrainingArguments(
    output_dir="./gpt2-gerchef",  # 模型训练输出目录
    overwrite_output_dir=True,  # 如果输出目录已存在，是否覆盖其内容
    num_train_epochs=3,  # 训练轮数，3轮
    per_device_train_batch_size=32,  # 每个设备（例如GPU）上的训练批量大小
    per_device_eval_batch_size=64,  # 每个设备上的评估批量大小
    eval_steps=400,  # 每 400 步进行一次评估
    save_steps=800,  # 每 800 步保存一次模型
    warmup_steps=500,  # 500步作为热身阶段，逐步提高学习率
)

def load_dataset(train_path, test_path, tokenizer):
    # 加载训练数据集
    train_dataset = TextDataset(
        tokenizer=tokenizer,
        file_path=train_path,
        block_size=128
    )

    # 加载测试数据集
    test_dataset = TextDataset(
        tokenizer=tokenizer,
        file_path=test_path,
        block_size=128
    )

    # 返回数据集和数据整理器
    return train_dataset, test_dataset, data_collator

# 6. 加载数据集
train_dataset, test_dataset, data_collator = load_dataset(train_path, test_path, tokenizer)

# 7. 创建 Trainer 实例，指定模型、训练参数、数据整理器、训练数据集和评估数据集
trainer = Trainer(
    model=model,  # 要训练的模型
    args=training_args,  # 训练参数
    data_collator=data_collator,  # 数据整理器
    train_dataset=train_dataset,  # 训练数据集，应该是一个 Dataset 对象（假设您已准备好）
    eval_dataset=test_dataset,  # 测试数据集，应该是一个 Dataset 对象（假设您已准备好）
)

# 8. 启动训练过程
trainer.train()  # 开始模型训练

trainer.save_model()